%% Licensed Materials - Property of IBM
%% Restricted Materials of IBM.  Reproduction, modification and redistribution are prohibited.
%% IBM Rational Rhapsody Design Manager
%% © Copyright IBM Corporation 2013  
%% U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

function [ minXY ] = getMinXYOfSimulinkModel( sys, parentSysPath )
    minXY = [];
    if isempty(sys) == 1
        return;
    end    

    minX = 0;
    minY = 0;
    try
        % first, traverse the model's blocks, and find the minXY position
        blks = get_param(sys, 'Blocks');
        sysName = get_param(sys, 'Name');
        if ~isempty(parentSysPath)
            sysName = horzcat(parentSysPath, '/', sysName);
        end    
        numOfBlocks = size(blks);
        for x = 1:numOfBlocks
            blockName = blks{x};
            tempBlockName = regexprep(blockName, '/', '//');
            fullBlockName = horzcat(sysName, '/', tempBlockName);
            blockPosition = getBlockPositionWithText(sys, blockName, fullBlockName);
            if minX == 0 || blockPosition(1) < minX
                minX = blockPosition(1);
            end
            if minY == 0 || blockPosition(2) < minY
                minY = blockPosition(2);
            end
        end 
    
        % next, traverse the model's lines, and find the minXY position
        lines = get_param(sys, 'Lines');
        numOfLines = size(lines);
        for x = 1:numOfLines
            line = lines(x);
            tmpMinXY = getMinXYOfLine(line);
            if isempty(tmpMinXY) == 0
                if(tmpMinXY(1) < minX)
                    minX = tmpMinXY(1);
                end
                if(tmpMinXY(2) < minY)
                    minY = tmpMinXY(2);
                end
            end              
        end

        % last, traverse the model's annotation, and find the minXY position
        annotationHandles = find_system(sys, 'FindAll', 'on', 'type', 'annotation');
        numOfAnnotations = size(annotationHandles);
        for x = 1:numOfAnnotations
            annotationPosition = getAnnotationPosition(sys, annotationHandles(x));
            if isempty(annotationPosition) == 0
                if minX == 0 || annotationPosition(1) < minX
                    minX = annotationPosition(1);
                end
                if minY == 0 || annotationPosition(2) < minY
                    minY = annotationPosition(2);
                end
            end     
        end
    catch e
        minX = 0;
        minY = 0;
    end
    
    minXY = [minX, minY];    
end


function [minXY] = getMinXYOfLine(line)
    minXY = [];
    minX = 0;
    minY = 0;
    if isempty(line) == 0
        linePoints = line.Points;
        ptsSize = size(linePoints);
        for i = 1 : ptsSize
            if minX == 0 || linePoints(i) < minX
                minX = linePoints(i, 1);
            end
            if minY == 0 || linePoints(i+1) < minY
                minY = linePoints(i, 2);
            end 
        end
        
        if isempty(line.Branch) == 0
            branchSize = size(line.Branch);
            for j = 1 : branchSize
                tmpMinXY = getMinXYOfLine(line.Branch(j));
                if isempty(tmpMinXY) == 0
                    if(tmpMinXY(1) < minX)
                        minX = tmpMinXY(1);
                    end
                    if(tmpMinXY(2) < minY)
                        minY = tmpMinXY(2);
                    end
                end 
            end    
        end
        minXY = [minX, minY];
    end    
end

function [textSize] = getTextSize(sys, blockName, fullBlockName, isAnnotation)
    textSize = [];
    if isempty(sys) == 0 && isempty(blockName) == 0 && isempty(fullBlockName) == 0
        if isAnnotation == 0
            showName = get_param(fullBlockName, 'ShowName');
            if isempty(showName) == 1 || strcmp(showName, 'off')
                return;
            end
        end    
        
                
        fontName = get_param(fullBlockName, 'FontName');
        if isempty(fontName) == 1 || strcmp(fontName, 'auto') == 1
            if isAnnotation == 1
                fontName = get_param(bdroot, 'DefaultAnnotationFontName');
            else    
                fontName = get_param(bdroot, 'DefaultBlockFontName'); 
            end    
        end 
        
        fontSize = get_param(fullBlockName, 'FontSize');
        if isempty(fontSize) == 1 || fontSize == -1
            if isAnnotation == 1
                fontSize = get_param(bdroot, 'DefaultAnnotationFontSize');
            else    
                fontSize = get_param(bdroot, 'DefaultBlockFontSize'); 
            end    
        end 
        
        fontWeightSize = java.awt.Font.PLAIN;
        fontWeight = get_param(fullBlockName, 'FontWeight');
        if isempty(fontWeight) == 1 || strcmp(fontWeight, 'auto') == 1
            if isAnnotation == 1
                fontWeight = get_param(bdroot, 'DefaultAnnotationFontWeight');
            else
                fontWeight = get_param(bdroot, 'DefaultBlockFontWeight'); 
            end    
        end
        fontAngle = get_param(fullBlockName, 'FontAngle');
        if isempty(fontAngle) == 1
            if isAnnotation == 1
                fontAngle = get_param(bdroot, 'DefaultAnnotationFontAngle');
            else
                fontAngle = get_param(bdroot, 'DefaultBlockFontAngle'); 
            end    
        end
        if strcmp(fontWeight, 'bold') == 1
            fontWeightSize = fontWeightSize + java.awt.Font.BOLD;
        end
        if strcmp(fontAngle, 'italic') == 1
            fontWeightSize = fontWeightSize + java.awt.Font.ITALIC;
        end
        
        largestLine = [];
        numOfLines = 0;
        if isempty(strfind(blockName, char(10))) == 0
            remain = blockName;
            while true
                [str, remain] = strtok(remain, char(10));
                if isempty(str) == 1
                    break;
                end
                numOfLines = numOfLines + 1;
                if length(char(str)) > length(char(largestLine))
                    largestLine = str;
                end    
            end
        else
            largestLine = blockName;
            numOfLines = 1;
        end
        
        textWidth = 0;
        textHeight = 0;
        if isempty(fontName) == 0 && isempty(fontSize) == 0
            try
                font = java.awt.Font(fontName, fontWeightSize, fontSize);
                img = java.awt.image.BufferedImage(1, 1, 2);
                fm = img.getGraphics().getFontMetrics(font);
                textWidth = fm.stringWidth(largestLine);
                textHeight = fm.getHeight() * numOfLines;
            catch e
                disp(e);
            end    
        end 
        textSize = [textWidth, textHeight];
    end    
end

function [position] = getBlockPositionWithText(sys, blockName, fullBlockName)
    position = [];
    if isempty(sys) == 0 && isempty(blockName) == 0 && isempty(fullBlockName) == 0
        position = get_param(fullBlockName, 'Position');
        textSize = getTextSize(sys, blockName, fullBlockName, 0);
        if isempty(textSize) == 0
            textWidth = textSize(1);
            textHeight = textSize(2);
            
            blkWidth = position(3) - position(1);
            blkHeight = position(4) - position(2);
            
            % 0 - no need to calculate
            % 1 - add only margin
            % 2 - add full text size
            bCalcX = 0;
            bCalcY = 0;
            blockRotation = get_param(fullBlockName, 'BlockRotation');
            namePlacement = get_param(fullBlockName, 'NamePlacement');
            if blockRotation == 0 && strcmp(namePlacement,'normal') == 1 
                bCalcX = 1;
                bCalcY = 0;
            elseif blockRotation == 0 && strcmp(namePlacement,'alternate') == 1
                bCalcX = 1;
                bCalcY = 2;    
            elseif blockRotation == 270 && strcmp(namePlacement,'normal') == 1
                bCalcX = 0;
                bCalcY = 1;    
            elseif blockRotation == 270 && strcmp(namePlacement,'alternate') == 1
                bCalcX = 2;
                bCalcY = 1;        
            end 
            
            if bCalcX == 1 && textWidth > blkWidth
                position(1) = position(1) - round((textWidth - blkWidth)/2);
            elseif bCalcX == 2
                position(1) = position(1) - textWidth;    
            end    
            
            if bCalcY == 1 && textHeight > blkHeight
                position(2) = position(2) - round((textHeight - blkHeight)/2);
            elseif bCalcY == 2
                position(2) = position(2) - textHeight;    
            end    
        end    
    end
end

function [position] = getAnnotationPosition(sys, annotationHandle)
    position = [];
    if isempty(sys) == 0 && isempty(annotationHandle) == 0
        text = get_param(annotationHandle, 'Text');
        hAlignment = get_param(annotationHandle, 'HorizontalAlignment');
        vAlignment = get_param(annotationHandle, 'VerticalAlignment');
        textSize = getTextSize(sys, text, annotationHandle, 1);
        if isempty(textSize) == 0
            position = get_param(annotationHandle, 'Position');
            x1 = position(1);
            y1 = position(2);
            textWidth = textSize(1);
            textHeight = textSize(2);
            
            if strcmp(hAlignment, 'center') == 1
                position(1) = x1 - (textWidth/2);
                position(3) = x1 + (textWidth/2);
            elseif strcmp(hAlignment, 'right') == 1
                position(1) = x1 - textWidth;
            end  
            
            if strcmp(vAlignment, 'middle') == 1
                position(2) = y1 - (textHeight/2);
                position(4) = y1 + (textHeight/2);
            elseif strcmp(vAlignment, 'bottom') == 1 
                position(2) = y1 - textHeight;
            end    
        end
    end    
end