%% Licensed Materials - Property of IBM
%% Restricted Materials of IBM.  Reproduction, modification and redistribution are prohibited.
%% IBM Rational Rhapsody Design Manager
%% © Copyright IBM Corporation 2013  
%% U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

function [ minXY ] = getMinXYOfStateFlowChart( chart )
    minXY = [];
    if isempty(chart) == 1
        return;
    end    

    minX = 0;
    minY = 0;

    try
        ch = '';
        if isa(chart, 'Stateflow.Chart') == 1 || ...
                ((isa(chart, 'Stateflow.Box') == 1 || isa(chart, 'Stateflow.Function') == 1 || ...
                 isa(chart, 'Stateflow.State') == 1) == 1 && chart.IsSubchart == 1) == 1
            ch = chart;
        else    
            chartName = get_param(chart, 'Name');
            root = sfroot;
            machine = root.find('-isa','Simulink.BlockDiagram');
            ch = machine.find('-isa','Stateflow.Chart', '-and', 'Name', chartName);
        end
        
        if isempty(ch) == 1
            return;
        end
        
        % first get all top level objects of the chart (i.e. States, Boxes, Function etc)
        objs = ch.find('-depth', 1);
        objs = setdiff(objs, ch);
        numOfObjs = size(objs);
        for i = 1 : numOfObjs
            obj = objs(i);
            if isa(obj, 'Stateflow.Data') == 1 || isa(obj, 'Stateflow.Event') == 1
                continue;
            end
            
            tmpMinX = 0;
            tmpMinY = 0;
            if isa(obj, 'Stateflow.Junction') == 1
                center = obj.Position.Center;
                radius = obj.Position.Radius;
                tmpMinX = center(1) - radius;
                tmpMinY = center(2) - radius;
                
            elseif isa(obj, 'Stateflow.Transition') == 1
                LabelPosition = obj.LabelPosition;
                SourceEndpoint = obj.SourceEndpoint;
                DestinationEndpoint = obj.DestinationEndpoint;
                tmpMinX = LabelPosition(1);
                if tmpMinX > SourceEndpoint(1);
                    tmpMinX = SourceEndpoint(1);
                end
                if tmpMinX > DestinationEndpoint(1);
                    tmpMinX = DestinationEndpoint(1);
                end    
                
                tmpMinY = LabelPosition(2);
                if tmpMinY > SourceEndpoint(2);
                    tmpMinY = SourceEndpoint(2);
                end
                if tmpMinY > DestinationEndpoint(2);
                    tmpMinY = DestinationEndpoint(2);
                end
            else
                % this must be a State, Box, Function, EMFunction, SLFunction, TruthTable, or note
                position = obj.Position;
                tmpMinX = position(1);
                tmpMinY = position(2);
            end
            
            if minX == 0 || tmpMinX < minX
                minX = tmpMinX;
            end  
            if minY == 0 || tmpMinY < minY
                minY = tmpMinY;
            end 
        end    
    catch
        minX = 0;
        minY = 0;
    end   
    
    minXY = [minX, minY]; 
end

